@@ -136,7 +136,7 @@ static int ttSyncDir( FILE* infoFile, FILE* nameFile, char* ttdir, int gslist)
 		strcat( pathName, de->d_name);
 
 		TTFont* ttFont = new TTFont( de->d_name, 1);
-		if( ttFont->badFont()) {
+		if( ttFont->badFont() || !ttFont) {
 			delete ttFont;
 			continue;
 		}
@@ -183,8 +183,8 @@ static int ttSyncDir( FILE* infoFile, FILE* nameFile, char* ttdir, int gslist)
 		delete ttFont;
 		++nfonts;
 	}
-
-	closedir( dirp);
+	if (dirp)
+		closedir( dirp);
 	return nfonts;
 }
 
@@ -470,10 +470,12 @@ static XFSFont* openFont( TTFont* ttFont, FontParams* fp,
 	raster->getFontExtent( &xfs->fe);
 
 	int used = (xfs->fe.bitmaps + xfs->fe.bmplen) - xfs->fe.buffer;
+	int bmpoff = xfs->fe.bitmaps - xfs->fe.buffer;
 	xfs->fe.buffer = (U8*)shrinkMem( xfs->fe.buffer, used);
-	if( xfs->fe.buffer)
+	if( xfs->fe.buffer) {
 		xfs->fe.buflen = used;
-	else {
+		xfs->fe.bitmaps = xfs->fe.buffer + bmpoff;
+	} else {
 		xfs->fid = 0;	//###
 		xfs = 0;
 	}
@@ -695,6 +697,12 @@ static int connecting( int sd)
 		return 0;
 
 	dprintf0( "Connecting\n");
+
+	if (req->byteOrder != 'l' && req->byteOrder != 'B') {
+		fputs("xfstt: invalid byteorder, giving up\n", stderr);
+		return 0;
+	}
+
 	dprintf1( "%s endian connection\n",
 		(req->byteOrder == 'l') ? "little" : "big");
 	dprintf2( "version %d.%d\n", req->major_version, req->minor_version);
@@ -817,6 +825,32 @@ static void fixup_bitmap( FontExtent* fe, U32 hint)
 	fe->bmpFormat = hint;
 }
 
+static int
+send_fserror(int sd, int seqno)
+{
+	fsError reply;
+
+	reply.type = FS_Error;
+	reply.request = FSBadLength;
+	reply.sequenceNumber = seqno;
+	reply.length = sizeof(reply) >> 2;
+
+	return write(sd, (void *)&reply, sizeof(reply));
+}
+
+static int
+check_length(int sd, int seqno, fsReq *req, int expected_size)
+{
+	if (req->length < (expected_size >> 2)) {
+		dprintf2("packet size mismatch: %d received bytes, "
+		      "%d expected bytes\n", req->length << 2, expected_size);
+		send_fserror(sd, seqno);
+		return 0;
+	} else {
+		return 1;
+	}
+}
+
 static int working( int sd, Rasterizer* raster, char* replybuf)
 {
 	FontParams fp0 = {{0,0,0,0}, {0,0,0,0}, {VGARES,VGARES}, 0}, fp;
@@ -839,15 +873,12 @@ static int working( int sd, Rasterizer* raster, char* replybuf)
 		sync();
 #endif
 
-		int length = ((fsReq*)buf)->length << 2;
+		fsReq *fsreq = (fsReq *)buf;
+		int length = fsreq->length << 2;
 		if( length > MAXREQSIZE) {
-			dprintf1( "request too big: %d bytes\n", length);
-			fsError reply;
-			reply.type		= FS_Error;
-			reply.request		= FSBadLength;
-			reply.sequenceNumber	= seqno;
-			reply.length		= sizeof(reply) >> 2;
-			write( sd, (void*)&reply, sizeof(reply));
+			dprintf2("too much data: %d bytes (max=%d)\n",
+			         length, MAXREQSIZE);
+			send_fserror(sd, seqno);
 			break;
 		}
 
@@ -990,7 +1021,13 @@ static int working( int sd, Rasterizer* raster, char* replybuf)
 		case FS_SetResolution:
 			{
 			fsSetResolutionReq* req = (fsSetResolutionReq*) buf;
-			int numres = req->num_resolutions;	//### 1
+			int numres = req->num_resolutions;
+			int expected_size = numres * sz_fsResolution
+					    + sz_fsSetResolutionReq;
+
+			if (!check_length(sd, seqno, fsreq, expected_size))
+				break;
+
 			dprintf1( "FS_SetResolution * %d\n", numres);
 			fsResolution* res = (fsResolution*)(req + 1);
 			for(; --numres >= 0; ++res) {
@@ -1033,6 +1070,11 @@ static int working( int sd, Rasterizer* raster, char* replybuf)
 			{
 			fsListFontsReq* req = (fsListFontsReq*) buf;
 			char* pattern = (char*) (req + 1);
+			int expected_size = sz_fsListFontsReq + req->nbytes;
+
+			if (!check_length(sd, seqno, fsreq, expected_size))
+				break;
+
 			pattern[ req->nbytes] = 0;
 			dprintf2( "FS_ListFonts \"%s\" * %ld\n",
 				pattern, req->maxNames);
@@ -1208,15 +1250,6 @@ dprintf2( "wmin= %d, wmax= %d)\n", fe->xAdvanceMin, fe->xAdvanceMax);
 			break;
 
 		case FS_QueryXExtents8:
-			// convert to QueryXExtents16 request
-			{
-			fsQueryXExtents8Req* req = (fsQueryXExtents8Req*) buf;
-			U8* p8 = (U8*)(req + 1);
-			U16* p16 = (U16*)p8;
-			for( i = req->num_ranges; --i >= 0;)
-				p16[ i] = htons( p8[ i]);
-			}
-			// fall through
 		case FS_QueryXExtents16:
 			{
 			fsQueryXExtents16Req* req = (fsQueryXExtents16Req*) buf;
@@ -1226,6 +1259,24 @@ dprintf2( "wmin= %d, wmax= %d)\n", fe->xAdvanceMin, fe->xAdvanceMax);
 			dprintf2( "range=%d, nranges=%ld\n",
 				req->range, req->num_ranges);
 
+			int item_size = (req->reqType == FS_QueryXExtents8)
+				        ? 1 : 2;
+			int expected_size = sz_fsQueryXExtents8Req
+				            + req->num_ranges * item_size;
+
+			if (!check_length(sd, seqno, fsreq, expected_size))
+				break;
+
+			if (req->reqType == FS_QueryXExtents8) {
+				/*
+				 * Convert to QueryXExtents16 request
+				 */
+				U8 *p8 = (U8 *)(req + 1);
+				U16 *p16 = (U16 *)p8;
+				for (i = req->num_ranges; --i >= 0;)
+					p16[i] = htons(p8[i]);
+			}
+
 			XFSFont* xfs = findFont( req->fid, sd, seqno);
 			if( !xfs)
 				break;
@@ -1294,15 +1345,6 @@ dprintf2( "wmin= %d, wmax= %d)\n", fe->xAdvanceMin, fe->xAdvanceMax);
 			break;
 
 		case FS_QueryXBitmaps8:
-			// convert to QueryXBitmaps16 request
-			{
-			fsQueryXBitmaps8Req* req = (fsQueryXBitmaps8Req*) buf;
-			U8* p8 = (U8*)(req + 1);
-			U16* p16 = (U16*)p8;
-			for( i = req->num_ranges; --i >= 0;)
-				p16[ i] = ntohs( p8[ i]);
-			}
-			// fall through
 		case FS_QueryXBitmaps16:
 			{
 			fsQueryXBitmaps16Req* req = (fsQueryXBitmaps16Req*) buf;
@@ -1311,6 +1353,24 @@ dprintf2( "wmin= %d, wmax= %d)\n", fe->xAdvanceMin, fe->xAdvanceMax);
 			dprintf2( "range=%d, nranges=%ld\n",
 				req->range, req->num_ranges);
 
+			int item_size = (req->reqType == FS_QueryXExtents8)
+				        ? 1: 2;
+			int expected_size = sz_fsQueryXBitmaps8Req
+					    + req->num_ranges * item_size;
+
+			if (!check_length(sd, seqno, fsreq, expected_size))
+				break;
+
+			if (req->reqType == FS_QueryXBitmaps8) {
+				/*
+				 * Convert to QueryXBitmaps16 request
+				 */
+				U8 *p8 = (U8 *)(req + 1);
+				U16 *p16 = (U16 *)p8;
+				for (i = req->num_ranges; --i >= 0;)
+					p16[i] = ntohs(p8[i]);
+			}
+
 			XFSFont* xfs = findFont( req->fid, sd, seqno);
 			if( !xfs)
 				break;
@@ -1692,12 +1752,17 @@ int main( int argc, char** argv)
 		close( sd);
 	} while( multiConnection);
 	
+	/* - it seemed like a good idea at the time. How else can we handle
+		the case where a user runs xfstt for a single x session and
+		then another user wants to use it later?
+ 
 	if (sockname) {
 		chdir( "/tmp/.font-unix");
 		unlink(sockname);
 		chdir("/tmp");
 		rmdir(".font-unix");
 	}
+	*/
 
 	dprintf0( "xfstt: closing a connection\n");
 	cleanupMem();
